﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using static System.Windows.Forms.VisualStyles.VisualStyleElement.Button;

namespace SVMctl_I2C_CSharp
{
    /// <summary>
    /// メイン画面
    /// </summary>
    public partial class FormSVMCtl : Form
    {
        #region ▼ Field
        /// <summary>
        /// 接続ボード情報
        /// </summary>
        private BoardInformation boardInformation;

        /// <summary>
        /// Setting File Write Dialogの初期設定ファイル名
        /// </summary>
        private string settingFileName;

        /// <summary>
        /// Log Export機能有効時のログファイル名（日付毎に追記作成）
        /// </summary>
        private string logFilePath = @"log\SVMCtl_Log_" + DateTime.Now.ToString().Substring(0, DateTime.Now.ToString().IndexOf(" ")).Replace("/", "") + ".csv";

        #endregion

        #region ▼ Property
        #endregion

        #region ▼ Constructor
        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="boardInformation">接続ボード情報</param>
        public FormSVMCtl(BoardInformation boardInformation = null)
        {
            // デザイナコンポーネントを初期化
            this.InitializeComponent();

            // 接続ボード情報代入
            this.boardInformation = boardInformation;
        }
        #endregion

        #region ▼ Event
        /// <summary>
        /// フォームが読み込まれるときの処理。
        /// </summary>
        /// <param name="sender">発生元</param>
        /// <param name="e">添付情報</param>
        private void FormSVMCtl_Load(object sender, EventArgs e)
        {
            
            // 保存された設定値を読み込む
            this.LoadSVMCtlSetting();
            
            // 接続ボード情報が無い場合
            if (this.boardInformation == null)
            {
                // タイトルを変更
                this.Text = "SVMCtl +[No Device]";

                // 一部のボタンを無効化
                this.buttonSettingFileWrite.Enabled = false;
                this.buttonWrite.Enabled = false;
                this.buttonRead.Enabled = false;
            }
            else
            {
                // タイトル文字列を作成
                StringBuilder title = new StringBuilder();
                title.Append("SVMCtl [");
                title.Append(this.boardInformation.BoardInfo);
                title.Append("] ");
                // USB3.0接続の場合
                if (this.boardInformation.IsUSB3 == true)
                {
                    title.Append("(USB3.0)");
                }
                else
                {
                    title.Append("(USB2.0)");
                }

                // タイトルを変更
                this.Text = title.ToString();
                
                // DWORD対応FWではない場合
                if( ! (BoardRevision.IS_SVM06(boardInformation.FwInfo.Revision) && (boardInformation.FwInfo.FWVersion >= 123)) ){
                    this.rb_dword_address.Enabled = false;
                    SetSelectedSubAddressMode(0);
                }
            }

            // Logファイル及びヘッダ作成
            if (!File.Exists(logFilePath))
            {
                this.checkBoxLogExport.Checked = true;
                ExportLog("Datetime,OperatioinMode,DeviceID,BoardType,BoardName,SlaveAddress,SubAddress,WordAccess,Data", false, true);
            }



            // ReadValueテキストボックスの折返しを無効化
            this.textBoxReadValue.WordWrap = false;

        }

        /// <summary>
        /// フォームが閉じられるときの処理
        /// </summary>
        /// <param name="sender">発生元</param>
        /// <param name="e">添付情報</param>
        private void FormSVMCtl_FormClosing(object sender, FormClosingEventArgs e)
        {
            // 設定値を保存する
            this.SaveSVMCtlSetting();
        }

        private int GetSelectedSubAddressMode()
        {
            var RadioButtonChecked_InGroup = gb_sub_address_mode.Controls.OfType<System.Windows.Forms.RadioButton>().SingleOrDefault(rb => rb.Checked == true);

            switch (RadioButtonChecked_InGroup.TabIndex)
            {
                default:
                case 0:
                    return 0;
                case 1:
                    return 1;
                case 2:
                    return 2;
            }
        }
        private void SetSelectedSubAddressMode(int selected)
        {
            // var RadioButtonChecked_InGroup = gb_sub_address_mode.Controls.OfType<System.Windows.Forms.RadioButton>().SingleOrDefault(rb => rb.Checked == true);

            switch (selected)
            {
                default:
                case 0:
                    rb_byte_address.Checked = true;
                    break;
                case 1:
                    rb_word_address.Checked = true;
                    break;
                case 2:
                    rb_dword_address.Checked = true;
                    break;
            }
        }

        /// <summary>
        /// SettingFileWriteボタンのクリックイベント。
        /// </summary>
        /// <param name="sender">発生元</param>
        /// <param name="e">添付情報</param>
        private void buttonSettingFileWrite_Click(object sender, EventArgs e)
        {
            // 設定ファイル情報代入変数
            UInt32 sfwSlaveAddress = 0;
            UInt32 sfwSubAddress;
            byte[] sfwDataByteArray;
            UInt32 sfwData;
            int sfwWordAddressMode = GetSelectedSubAddressMode();
            int sfwUnit = 16;
            StringBuilder readValue = new StringBuilder();
            uint readCounts;

            //ExportLog("[buttonSettingFileWrite_Click]");

            // Logファイル及びヘッダ作成
            if (!File.Exists(logFilePath))
            {
                this.checkBoxLogExport.Checked = true;
                ExportLog("Datetime,OperatioinMode,DeviceID,BoardType,BoardName,SlaveAddress,SubAddress,WordAccess,Data", false, true);
            }

            // ログファイル書き込みチェック
            if (this.checkBoxLogExport.Checked && !LogFileAccesability(logFilePath))
            {
                MessageBox.Show("ログファイル書き込みエラー\nファイルを閉じて再実行して下さい。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }


            // SlaveAddress初期値設定
            if (this.textBoxI2CSlaveAddress.Text.Trim() != "")
            {
                sfwSlaveAddress = Convert.ToUInt32((this.textBoxI2CSlaveAddress.Text).Trim(), 16);
            }

            // ファイルを開くダイアログ
            OpenFileDialog openFileDialog = new OpenFileDialog();
            // ファイル名
            openFileDialog.FileName = this.settingFileName;
            // 「ファイルの種類」の選択肢
            openFileDialog.Filter = "Text and Initialize Files|*.txt|CSV Files|*.csv|All Files|*.*";
            // ダイアログのタイトル
            openFileDialog.Title = "Open FIle";

            // ファイルを開くダイアログを表示し、キャンセルされた場合
            if (DialogResult.OK != openFileDialog.ShowDialog())
            {
                // 以降の処理はしない
                return;
            }

            // 開いた設定ファイル名をiniファイルの参照先に代入
            this.settingFileName = openFileDialog.FileName;

            // デバイス初期化
            SVIControl.DeviceOpen(boardInformation.DeviceId, boardInformation.BoardType);

            // ボタン無効化
            this.buttonSettingFileWrite.Enabled = false;
            this.buttonRead.Enabled = false;
            this.buttonWrite.Enabled = false;

            // ファイル読み込み
            using (StreamReader reader = new StreamReader(openFileDialog.FileName, Encoding.GetEncoding("Shift_JIS")))
            {
                string line;
                int lineCount = 0;
                int waitValue = 0;


                // １行ずつ読み込む
                while ((line = reader.ReadLine()) != null)
                {
                    lineCount++;

                    // 左右空白削除, 大文字変換
                    line = line.Trim().ToUpper();

                    // 空白行スキップ
                    if (String.IsNullOrWhiteSpace(line))
                    {
                        continue;
                    }

                    // コメント行スキップ
                    if (line.StartsWith(";") || line.StartsWith("#") || line.StartsWith("/"))
                    {
                        continue;
                    }

                    // 行間に含まれるコメントを削除
                    if (line.Contains(";"))
                    {
                        line = line.Substring(0, line.IndexOf(";"));
                    }
                    if (line.Contains("#"))
                    {
                        line = line.Substring(0, line.IndexOf("#"));
                    }
                    if (line.Contains("/"))
                    {
                        line = line.Substring(0, line.IndexOf("/"));
                    }

                    // 命令詞UNIT行の不要文字列チェック
                    if (line.StartsWith("UNIT"))
                    {
                        // 基数抽出, エラーチェック, sfwUnitへの代入
                        if (((int.TryParse(line.Replace("UNIT", "").Replace(",", "").Trim(), out sfwUnit) == true) && (sfwUnit == 10 || sfwUnit == 16)) == true)
                        {
                            continue;
                        }
                    }

                    // wait処理
                    if (line.StartsWith("WT"))
                    {
                        if (sfwUnit == 16)
                        {
                            if (int.TryParse(line.Replace("WT", "").Replace(",", "").Trim(), System.Globalization.NumberStyles.HexNumber, null, out waitValue))
                            {
                                Thread.Sleep(waitValue);
                                continue;
                            }
                            else
                            {
                                break;
                            }
                        }
                        else
                        {
                            if (int.TryParse(line.Replace("WT", "").Replace(",", "").Trim(), out waitValue))
                            {
                                Thread.Sleep(waitValue);
                                continue;
                            }
                            else
                            {
                                break;
                            }
                        }
                    }

                    // 命令詞BYTE行の不要文字列チェック
                    if (line == "BYTE")
                    {
                        sfwWordAddressMode = 0;
                        continue;
                    }

                    // 命令詞WORD行の不要文字列チェック
                    if (line == "WORD")
                    {
                        sfwWordAddressMode = 1;
                        continue;
                    }
                    
                    // 命令詞DWORD行の不要文字列チェック
                    if (line == "DWORD")
                    {
                        sfwWordAddressMode = 2;
                        continue;
                    }

                    // 命令詞SLAVE行の不要文字列チェック
                    if (line.StartsWith("SLAVE"))
                    {
                        // エラーチェックとsfwSlaveAddressへの代入
                        if (this.IsHex(line.Replace("SLAVE", "").Replace(",", "").Trim()))
                        {
                            sfwSlaveAddress = Convert.ToUInt32(line.Replace("SLAVE", "").Replace(",", "").Trim(), sfwUnit);
                            continue;
                        }
                    }

                    // I2C Read実行
                    if (line.Contains("READ"))
                    {
                        readValue = new StringBuilder();
                        line = line.Replace("READ,", "").Replace("READ", "");

                        // 区切り文字スペースをコンマに置換
                        if (line.Trim().Contains(" "))
                        {
                            line = line.Trim().Replace(" ", ",");
                        }
                        else if (line.Trim().Contains("\t"))
                        {
                            line = line.Trim().Replace("\t", ",");
                        }

                        // UInt32に変換可能な場合、変換後代入
                        if (UInt32.TryParse(line.Substring(0, line.IndexOf(",")), System.Globalization.NumberStyles.HexNumber, null, out sfwSubAddress))
                        {
                            sfwSubAddress = Convert.ToUInt32(line.Substring(0, line.IndexOf(",")), sfwUnit);
                        }
                        else
                        {
                            continue;
                        }

                        // 通信実行
                        if (sfwSlaveAddress == 8)
                        {
                            // FPGAレジスタ読み出しを実行(SPI)
                            ExportLog("[buttonSettingFileWrite_Click] FPGA Register Read,DeviceId:" + this.boardInformation.DeviceId + ",BoardType:" + this.boardInformation.BoardType + ",BoardName:" + this.boardInformation.BoardName
                                + ",SlaveAddress:0x00000008" + ",RegisterAddress:0x" + sfwSubAddress.ToString("x8") + ",", true, false);

                            UInt32? receiveBuffer = SVIControl.FpgaRegisterRead(this.boardInformation.DeviceId, this.boardInformation.BoardType, sfwSubAddress, false, false);

                            // データの読み出しに失敗した場合
                            if (receiveBuffer == null)
                            {
                                // エラーメッセージBOX
                                MessageBox.Show("Error:\n" + lineCount + ": " + line, "Settings File Write Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                MessageBox.Show("FPGAレジスタの読込に失敗しました。\nボードとの接続や設定内容を確認してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                ExportLog("", false, true);
                                ExportLog("[buttonSettingFileWrite_Click] FPGA Register Read Error.");

                                // 以降の処理はしない
                                break;
                            }

                            // 表示用文字列整形
                            readValue.Append(((uint)receiveBuffer).ToString("X8").Insert(2, ",").Insert(5, ",").Insert(8, ",").Insert(11, ","));
                        }
                        else
                        {
                            // Read Count取得
                            // UInt32に変換可能な場合、変換後代入
                            if (UInt32.TryParse(line.Substring(line.IndexOf(",") + 1), out readCounts))
                            {
                                readCounts = Convert.ToUInt32(line.Substring(line.IndexOf(",") + 1), 10);
                            }
                            else
                            {
                                continue;
                            }

                            // I2Cデータ読み出しを実行
                            ExportLog("[buttonSettingFileWrite_Click] I2C Block Read,DeviceId:" + this.boardInformation.DeviceId + ",BoardType:" + this.boardInformation.BoardType + ",BoardName:" + this.boardInformation.BoardName
                                + ",SlaveAddress:0x" + sfwSlaveAddress.ToString("x8") + ",SubAddress:0x" + sfwSubAddress.ToString("x8")
                                + ",WordAccess:" + sfwWordAddressMode, true, false);

                            byte[] dataArray = SVIControl.I2C_BlockRead(this.boardInformation.DeviceId, this.boardInformation.BoardType,
                                sfwSlaveAddress, sfwSubAddress, readCounts, sfwWordAddressMode, false);

                            // データの読み出しに失敗した場合
                            if (dataArray == null || dataArray.Length == 0)
                            {
                                // エラーメッセージBOX
                                MessageBox.Show("Error:\n" + lineCount + ": " + line, "Settings File Write Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                MessageBox.Show("カメラレジスタの読込に失敗しました。\nボードとの接続や設定内容を確認してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                ExportLog("", false, true);
                                ExportLog("[buttonSettingFileWrite_Click] I2C Block Read Error.");

                                // 以降の処理はしない
                                break;
                            }

                            // 表示用文字列整形
                            for (int i = 0; i < readCounts; i++)
                            {
                                // 16進数2桁
                                readValue.Append(dataArray[i].ToString("X2"));
                                readValue.Append(",");
                            }
                        }

                        // 末尾のカンマを削除
                        readValue.Length--;
                        readValue.AppendLine();

                        // ReadValueログ出力
                        if (sfwSlaveAddress == 8)
                        {
                            ExportLog(",ReadRegisterValue:0x" + readValue.ToString().ToLower().Replace("\r", "").Replace("\n", "").Replace(",", ""), false, true);
                        }
                        else
                        {
                            ExportLog(",ReadData:0x" + readValue.ToString().ToLower().Replace("\r", "").Replace("\n", "").Replace(",", " 0x"), false, true);
                        }

                        continue;
                    }

                    // I2C Write実行 (区切り文字"," " " "\t")
                    if (line.Contains(",") || line.Trim().Contains(" ") || line.Trim().Contains("\t"))
                    {
                        // 区切り文字スペースをコンマに置換
                        if (line.Trim().Contains(" "))
                        {
                            line = line.Trim().Replace(" ", ",");
                        }
                        else if (line.Trim().Contains("\t"))
                        {
                            line = line.Trim().Replace("\t", ",");
                        }

                        // UInt32に変換可能な場合、変換後代入
                        if (UInt32.TryParse(line.Substring(0, line.IndexOf(",")), System.Globalization.NumberStyles.HexNumber, null, out sfwSubAddress))
                        {
                            sfwSubAddress = Convert.ToUInt32(line.Substring(0, line.IndexOf(",")), sfwUnit);
                        }

                        if (sfwSlaveAddress == 0x8)
                        {
                            sfwDataByteArray = this.StringToByteArray(line.Substring(line.IndexOf(",") + 1, line.Length - (line.IndexOf(",") + 1)), sfwUnit);
                            // Byte配列をUInt32型に格納
                            sfwData = (UInt32)(sfwDataByteArray[0] << 8 * 3 | sfwDataByteArray[1] << 8 * 2 | sfwDataByteArray[2] << 8 * 1 | sfwDataByteArray[3]);

                            // FPGAレジスタ書き込み
                            ExportLog("[buttonSettingFileWrite_Click] FPGA Register Write,DeviceId:" + this.boardInformation.DeviceId + ",BoardType:" + this.boardInformation.BoardType + ",BoardName:" + this.boardInformation.BoardName
                                + ",SlaveAddress:0x00000008" + ",RegisterAddress:0x" + sfwSubAddress.ToString("x8") + ",,WriteRegisterValue:0x" + sfwData.ToString("x8"));

                            if (SVIControl.FpgaRegisterWrite(this.boardInformation.DeviceId, this.boardInformation.BoardType, sfwSubAddress, sfwData, false, false) == NativeMethods.RET_NORMAL)
                            {
                                continue;
                            }
                            else
                            {
                                // エラーメッセージBOX
                                MessageBox.Show("Error:\n" + lineCount + ": " + line, "Settings File Write Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                MessageBox.Show("FPGAレジスタの読込に失敗しました。\nボードとの接続や設定内容を確認してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                ExportLog("[buttonSettingFileWrite_Click] Fpga Register Write Error");

                                break;
                            }
                        }
                        else
                        {
                            sfwDataByteArray = StringToByteArray(line.Substring(line.IndexOf(",") + 1, line.Length - (line.IndexOf(",") + 1)), sfwUnit);

                            // I2C書き込み
                            string data = "";
                            foreach (byte d in sfwDataByteArray)
                            {
                                data += "0x" + d.ToString("x2") + " ";
                            }

                            ExportLog("[buttonSettingFileWrite_Click] I2C Block Write,DeviceId:" + this.boardInformation.DeviceId + ",BoardType:" + this.boardInformation.BoardType + ",BoardName:" + this.boardInformation.BoardName
                                + ",SlaveAddress:0x" + sfwSlaveAddress.ToString("x8") + ",SubAddress:0x" + sfwSubAddress.ToString("x8")
                                + ",WordAccess:" + sfwWordAddressMode + ",WriteData:" + data.Trim());

                            if (SVIControl.I2C_BlockWrite(this.boardInformation.DeviceId, this.boardInformation.BoardType, sfwSlaveAddress, sfwSubAddress, sfwDataByteArray, sfwWordAddressMode, false) == NativeMethods.RET_NORMAL)
                            {
                                continue;
                            }
                            else
                            {
                                // エラーメッセージBOX
                                MessageBox.Show("Error:\n" + lineCount + ": " + line, "Settings File Write Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                MessageBox.Show("カメラレジスタの読込に失敗しました。\nボードとの接続や設定内容を確認してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                ExportLog("[buttonSettingFileWrite_Click] I2C Block Write Error");

                                break;
                            }
                        }
                    }

                    // エラーメッセージBOX
                    MessageBox.Show("Error:\n" + lineCount + ": " + line, "Settings File Write Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    ExportLog("[buttonSettingFileWrite_Click] Settings File Write Error Line:" + lineCount + ":" + line);

                    break;
                }
            }

            // デバイス終了処理
            SVIControl.DeviceClose();

            // ボタン有効化
            this.buttonSettingFileWrite.Enabled = true;
            this.buttonRead.Enabled = true;
            this.buttonWrite.Enabled = true;
        }

        /// <summary>
        /// Writeボタンのクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonWrite_Click(object sender, EventArgs e)
        {
            uint i2cSlaveAddress;
            uint i2cSubAddress;
            byte[] writeValueByteArray;
            UInt32 writeValue;
            string data = "";

            #region 入力値チェック
            // Slave Address
            string i2cSlaveAddressText = this.textBoxI2CSlaveAddress.Text.Trim();

            //ExportLog("[buttonWrite_Click]");

            // Logファイル及びヘッダ作成
            if (!File.Exists(logFilePath))
            {
                this.checkBoxLogExport.Checked = true;
                ExportLog("Datetime,OperatioinMode,DeviceID,BoardType,BoardName,SlaveAddress,SubAddress,WordAccess,Data", false, true);
            }

            // ログファイル書き込みチェック
            if (this.checkBoxLogExport.Checked && !LogFileAccesability(logFilePath))
            {
                MessageBox.Show("ログファイル書き込みエラー\nファイルを閉じて再実行して下さい。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // I2CSlaveAddress欄が空欄の場合
            if (string.IsNullOrWhiteSpace(i2cSlaveAddressText) == true)
            {
                MessageBox.Show("I2C Slave Addressが未設定です。設定してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                ExportLog("[buttonWrite_Click] Slave Address Error.");

                // I2CSlaveAddress欄に移動
                this.textBoxI2CSlaveAddress.Focus();

                // 以降の処理はしない
                return;
            }

            // I2CSlaveAddress欄の入力値が16進数以外の場合
            if (this.IsHex(i2cSlaveAddressText) == false)
            {
                MessageBox.Show("Slave Addressに16進数以外が設定されてます。再設定してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                ExportLog("[buttonWrite_Click] Slave Address Error.");

                // I2CSlaveAddress欄に移動
                this.textBoxI2CSlaveAddress.Focus();

                // 以降の処理はしない
                return;
            }
            else
            {
                i2cSlaveAddress = Convert.ToUInt32(i2cSlaveAddressText, 16);
            }


            // Sub Address
            string i2cSubAddressText = this.textBoxSubAddress.Text.Trim();

            // I2CSubAddress欄が空欄の場合
            if (string.IsNullOrWhiteSpace(i2cSubAddressText) == true)
            {
                MessageBox.Show("Sub Addressが未設定です。設定してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                ExportLog("[buttonWrite_Click] Sub Address Error.");

                // I2CSubAddress欄に移動
                this.textBoxSubAddress.Focus();

                // 以降の処理はしない
                return;
            }

            // SubAddress欄の入力値が16進数以外の場合
            if (this.IsHex(i2cSubAddressText) == false)
            {
                MessageBox.Show("Sub Addressに16進数以外が設定されてます。再設定してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                ExportLog("[buttonWrite_Click] Sub Address Error.");

                // SubAddress欄に移動
                this.textBoxSubAddress.Focus();

                // 以降の処理はしない
                return;
            }
            else
            {
                i2cSubAddress = Convert.ToUInt32(i2cSubAddressText, 16);
            }


            // Write Value確認

            // WriteValue欄が空欄の場合
            if (string.IsNullOrWhiteSpace(this.textBoxWriteValue.Text) == true)
            {
                MessageBox.Show("Write Valueが未設定です。設定してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                ExportLog("[buttonWrite_Click] Write Value Error.");

                // WriteValue欄に移動
                this.textBoxWriteValue.Focus();

                // 以降の処理はしない
                return;
            }

            // WriteValue欄の入力値が16進数以外の場合
            if (this.IsHex(this.textBoxWriteValue.Text.Replace(",", "")) == false)
            {
                MessageBox.Show("Write Valueに16進数以外が設定されてます。再設定してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                ExportLog("[buttonWrite_Click] Write Value Error.");

                // SubAddress欄に移動
                this.textBoxWriteValue.Focus();

                // 以降の処理はしない
                return;
            }



            #endregion 入力値チェック

            if (i2cSlaveAddress == 0x8)
            {
                if (this.textBoxWriteValue.Text.Contains(","))
                {
                    writeValueByteArray = this.StringToByteArray(this.textBoxWriteValue.Text, 16);

                    // Byte配列をUInt32型に格納
                    writeValue = (UInt32)(writeValueByteArray[0] << 8 * 3 | writeValueByteArray[1] << 8 * 2 | writeValueByteArray[2] << 8 * 1 | writeValueByteArray[3]);
                }
                else
                {
                    writeValue = Convert.ToUInt32(this.textBoxWriteValue.Text, 16);
                }

                // Slave Address == 0x08の場合、FPGAレジスタに対しSPI通信を実行
                ExportLog("[buttonWrite_Click] FPGA Register Write,DeviceId:" + this.boardInformation.DeviceId + ",BoardType:" + this.boardInformation.BoardType + ",BoardName:" + this.boardInformation.BoardName
                    + ",SlaveAddress:0x00000008" + ",RegisterAddress:0x" + i2cSubAddress.ToString("x8") + ",,WriteRegisterValue:0x" + writeValue.ToString("x8"));

                if (SVIControl.FpgaRegisterWrite(this.boardInformation.DeviceId, this.boardInformation.BoardType,
                    i2cSubAddress, writeValue) == null)
                {
                    // FPGAレジスタアクセスに失敗(通信APIの戻り値が異常値)の場合
                    MessageBox.Show("FPGAレジスタへの設定に失敗しました。\nWrite Valueやボードとの接続を確認してください。",
                        "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    ExportLog("[buttonWrite_Click] FPGA Register Write Error.");
                }
            }
            else
            {
                writeValueByteArray = this.StringToByteArray(this.textBoxWriteValue.Text);

                // StringToByteArray関数の戻り値がnullの場合 (Write Valueの値が1byteを超える場合)
                if (writeValueByteArray == null)
                {
                    MessageBox.Show("Write Valueの値が不正です。確認してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    ExportLog("[buttonWrite_Click] Write Value Error.");

                    // WriteValue欄に移動
                    this.textBoxWriteValue.Focus();

                    // 以降の処理はしない
                    return;
                }

                // I2C通信を実行
                foreach (byte d in writeValueByteArray)
                {
                    data += "0x" + d.ToString("x2") + " ";
                }

                ExportLog("[buttonWrite_Click] I2C Block Write,DeviceId:" + this.boardInformation.DeviceId + ",BoardType:" + this.boardInformation.BoardType + ",BoardName:" + this.boardInformation.BoardName
                    + ",SlaveAddress:0x" + i2cSlaveAddress.ToString("x8") + ",SubAddress:0x" + i2cSubAddress.ToString("x8")
                    + ",WordAccess:" + GetSelectedSubAddressMode() + ",WriteData:" + data.Trim());

                // データの書き込みに失敗した場合
                if (SVIControl.I2C_BlockWrite(this.boardInformation.DeviceId, this.boardInformation.BoardType,
                    i2cSlaveAddress, i2cSubAddress, writeValueByteArray, GetSelectedSubAddressMode()) == null)
                {
                    // I2C通信に失敗(通信APIの戻り値が異常値)の場合
                    MessageBox.Show("カメラレジスタへの設定に失敗しました。\n設定ファイルの内容、カメラやボードとの接続を確認してください。",
                        "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    ExportLog("[buttonWrite_Click] I2C Block Write Error.");
                }
            }
        }

        /// <summary>
        /// Readボタンのクリックイベント
        /// </summary>
        /// <param name="sender">発生元</param>
        /// <param name="e">添付情報</param>
        private void buttonRead_Click(object sender, EventArgs e)
        {
            uint i2cSlaveAddress;
            uint i2cSubAddress;
            uint readCounts;

            #region 入力値チェック

            // Slave Address
            string i2cSlaveAddressText = this.textBoxI2CSlaveAddress.Text.Trim();

            // I2CSlaveAddress欄が空欄の場合
            if (string.IsNullOrWhiteSpace(i2cSlaveAddressText) == true)
            {
                MessageBox.Show("I2C Slave Addressが未設定です。設定してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                // I2CSlaveAddress欄に移動
                this.textBoxI2CSlaveAddress.Focus();

                // 以降の処理はしない
                return;
            }

            // I2CSlaveAddress欄の入力値が16進数以外の場合
            if (this.IsHex(i2cSlaveAddressText) == false)
            {
                MessageBox.Show("I2C Slave Addressに16進数以外が設定されてます。再設定してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                // I2CSlaveAddress欄に移動
                this.textBoxI2CSlaveAddress.Focus();

                // 以降の処理はしない
                return;
            }
            else
            {
                i2cSlaveAddress = Convert.ToUInt32(i2cSlaveAddressText, 16);
            }


            // Sub Address
            string i2cSubAddressText = this.textBoxSubAddress.Text.Trim();

            // I2CSubAddress欄が空欄の場合
            if (string.IsNullOrWhiteSpace(i2cSubAddressText) == true)
            {
                MessageBox.Show("Sub Addressが未設定です。設定してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                // I2CSlaveAddress欄に移動
                this.textBoxSubAddress.Focus();

                // 以降の処理はしない
                return;
            }

            // SubAddress欄の入力値が16進数以外の場合
            if (this.IsHex(i2cSubAddressText) == false)
            {
                MessageBox.Show("Sub Addressに16進数以外が設定されてます。再設定してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                // SubAddress欄に移動
                this.textBoxSubAddress.Focus();

                // 以降の処理はしない
                return;
            }
            else
            {
                i2cSubAddress = Convert.ToUInt32(i2cSubAddressText, 16);
            }


            // Read Counts
            // ReadCounts欄に英数字以外が入力されている場合
            if (uint.TryParse(this.textBoxReadCounts.Text.Trim(), out readCounts) == false)
            {
                MessageBox.Show("Read Countsに数値以外が設定されてます。再設定してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                // ReadCounts欄に移動
                this.textBoxReadCounts.Focus();

                // 以降の処理はしない
                return;
            }
            #endregion


            //ExportLog("[buttonRead_Click]");

            // Logファイル及びヘッダ作成
            if (!File.Exists(logFilePath))
            {
                this.checkBoxLogExport.Checked = true;
                ExportLog("Datetime,OperatioinMode,DeviceID,BoardType,BoardName,SlaveAddress,SubAddress,WordAccess,Data", false, true);
            }

            // ログファイル書き込みチェック
            if (this.checkBoxLogExport.Checked && !LogFileAccesability(logFilePath))
            {
                MessageBox.Show("ログファイル書き込みエラー\nファイルを閉じて再実行して下さい。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // 表示用文字列を定義
            StringBuilder readValue = new StringBuilder();

            // 通信実行
            if (i2cSlaveAddress == 8)
            {
                // FPGAレジスタ読み出しを実行(SPI)
                ExportLog("[buttonRead_Click] FPGA Register Read,DeviceId:" + this.boardInformation.DeviceId + ",BoardType:" + this.boardInformation.BoardType + ",BoardName:" + this.boardInformation.BoardName
                    + ",SlaveAddress:0x00000008" + ",RegisterAddress:0x" + i2cSubAddress.ToString("x8") + ",", true, false);

                UInt32? receiveBuffer = SVIControl.FpgaRegisterRead(this.boardInformation.DeviceId, this.boardInformation.BoardType, i2cSubAddress);

                // データの読み出しに失敗した場合
                if (receiveBuffer == null)
                {
                    MessageBox.Show("FPGAレジスタの読込に失敗しました。\nボードとの接続や設定内容を確認してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    ExportLog("", false, true);
                    ExportLog("[buttonRead_Click] FPGA Register Read Error.");

                    // 以降の処理はしない
                    return;
                }

                // 表示用文字列整形
                readValue.Append(((uint)receiveBuffer).ToString("X8").Insert(2, ",").Insert(5, ",").Insert(8, ",").Insert(11, ","));


            }
            else
            {
                // I2Cデータ読み出しを実行
                ExportLog("[buttonRead_Click] I2C Block Read,DeviceId:" + this.boardInformation.DeviceId + ",BoardType:" + this.boardInformation.BoardType + ",BoardName:" + this.boardInformation.BoardName
                    + ",SlaveAddress:0x" + i2cSlaveAddress.ToString("x8") + ",SubAddress:0x" + i2cSubAddress.ToString("x8")
                    + ",WordAccess:" + GetSelectedSubAddressMode(), true, false);

                byte[] dataArray = SVIControl.I2C_BlockRead(this.boardInformation.DeviceId, this.boardInformation.BoardType,
                    i2cSlaveAddress, i2cSubAddress, readCounts, GetSelectedSubAddressMode());

                // データの読み出しに失敗した場合
                if (dataArray == null || dataArray.Length == 0)
                {
                    MessageBox.Show("カメラレジスタの読込に失敗しました。\n設定ファイルの内容、カメラやボードとの接続を確認してください。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    ExportLog("", false, true);
                    ExportLog("[buttonRead_Click] I2C Block Read Error.");

                    // 以降の処理はしない
                    return;
                }

                // 表示用文字列整形
                for (int i = 0; i < readCounts; i++)
                {
                    // 16進数2桁
                    readValue.Append(dataArray[i].ToString("X2"));
                    readValue.Append(",");
                }
            }


            // 末尾のカンマを削除
            readValue.Length--;
            readValue.AppendLine();

            // ReadValue欄に追記
            this.textBoxReadValue.Text += readValue.ToString();

            // ReadValue欄内最終行に移動
            this.textBoxReadValue.SelectionStart = this.textBoxReadValue.Text.Length;
            this.textBoxReadValue.ScrollToCaret();

            // ReadValueログ出力
            if (i2cSlaveAddress == 8)
            {
                ExportLog(",ReadRegisterValue:0x" + readValue.ToString().ToLower().Replace("\r", "").Replace("\n", "").Replace(",", ""), false, true);
            }
            else
            {
                ExportLog(",ReadData:0x" + readValue.ToString().ToLower().Replace("\r", "").Replace("\n", "").Replace(",", " 0x"), false, true);
            }
        }

        /// <summary>
        /// Clearボタンのクリックイベント
        /// </summary>
        /// <param name="sender">発生元</param>
        /// <param name="e">添付情報</param>
        private void buttonClear_Click(object sender, EventArgs e)
        {
            this.textBoxReadValue.Text = "";

            // Logファイル及びヘッダ作成
            if (!File.Exists(logFilePath))
            {
                this.checkBoxLogExport.Checked = true;
                ExportLog("Datetime,OperatioinMode,DeviceID,BoardType,BoardName,SlaveAddress,SubAddress,WordAccess,Data", false, true);
            }

            // ログファイル書き込みチェック
            if (this.checkBoxLogExport.Checked && !LogFileAccesability(logFilePath))
            {
                MessageBox.Show("ログファイル書き込みエラー\nファイルを閉じて再実行して下さい。", "SVMctl", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            ExportLog("[buttonClear_Click]");
        }

        /// <summary>
        /// テキストボックスがアクティブになったときの共通処理
        /// </summary>
        /// <param name="sender">発生元</param>
        /// <param name="e">添付情報</param>
        private void TextBox_Enter(object sender, EventArgs e)
        {
            // テキストを全選択
            ((TextBox)sender).SelectAll();
        }
        #endregion

        #region ▼ Private Method
        /// <summary>
        /// iniファイルに保存された設定値を読み込み
        /// </summary>
        private void LoadSVMCtlSetting()
        {
            // iniファイルに保存された設定値を画面へ反映
            this.textBoxI2CSlaveAddress.Text = IniFileController.GetString(IniFileController.Keys.SlaveAddress);
            this.textBoxSubAddress.Text = IniFileController.GetString(IniFileController.Keys.SubAddress);
            this.textBoxWriteValue.Text = IniFileController.GetString(IniFileController.Keys.WriteValue);
            this.textBoxReadCounts.Text = IniFileController.GetString(IniFileController.Keys.ReadCounts);
            SetSelectedSubAddressMode(IniFileController.GetInt(IniFileController.Keys.WordAddressMode));
            this.settingFileName = IniFileController.GetString(IniFileController.Keys.SettingFileName);
            this.checkBoxLogExport.Checked = IniFileController.GetBool(IniFileController.Keys.ExportLog);

        }

        /// <summary>
        /// iniファイルへ設定値を保存
        /// </summary>
        private void SaveSVMCtlSetting()
        {
            IniFileController.WriteString(IniFileController.Keys.SlaveAddress, this.textBoxI2CSlaveAddress.Text);
            IniFileController.WriteString(IniFileController.Keys.SubAddress, this.textBoxSubAddress.Text);
            IniFileController.WriteString(IniFileController.Keys.WriteValue, this.textBoxWriteValue.Text);
            IniFileController.WriteString(IniFileController.Keys.ReadCounts, this.textBoxReadCounts.Text);
            IniFileController.WriteInt(IniFileController.Keys.WordAddressMode, GetSelectedSubAddressMode());
            IniFileController.WriteString(IniFileController.Keys.SettingFileName, this.settingFileName);
            IniFileController.WriteBool(IniFileController.Keys.ExportLog, this.checkBoxLogExport.Checked);
        }

        /// <summary>
        /// 文字列が16進数に変換可能かどうかを検証
        /// </summary>
        /// <param name="target">検証対象の文字列</param>
        /// <returns>true..16進数に変換可能、false..16進数に変換不可</returns>
        private bool IsHex(string target)
        {
            target = target.Trim().ToLower().Replace("0x", "");

            // 空欄なら変換不可
            if (string.IsNullOrWhiteSpace(target) == true)
            {
                return false;
            }

            // 16進数文字でない文字が含まれるなら変換不可
            foreach (char c in target)
            {
                if (Uri.IsHexDigit(c) == false)
                {
                    return false;
                }
            }

            return true;
        }


        /// <summary>
        /// string型文字列をByte型配列に変換
        /// </summary>
        /// <param name="stringData">変換元string型文字列</param>
        /// <returns>正常時:変換後のByte型配列 異常時:null</returns>
        private byte[] StringToByteArray(string stringData, int conversionUnit = 16)
        {
            // 区切り文字
            char[] delimiter = { ',', ' ' };

            // 対象文字列が空の場合
            if (string.IsNullOrWhiteSpace(stringData) == true)
            {
                return null;
            }

            // 前後空白除去
            stringData = stringData.Trim();

            // 対象文字列を分割して取得(空白要素は除去)
            string[] stringDataArray = stringData.Split(delimiter, StringSplitOptions.RemoveEmptyEntries);

            byte[] byteArray = new byte[stringDataArray.Length];

            for (int i = 0; i < stringDataArray.Length; i++)
            {
                UInt32 temp;

                // Write Valueに"0x FF"等の不正な値が入力された場合の例外処理
                if (this.IsHex(stringDataArray[i]) == true)
                {
                    temp = Convert.ToUInt32(stringDataArray[i], conversionUnit);
                }
                else
                {
                    return null;
                }

                // 1byte超でnullを返す
                if (temp > 0xFF)
                {
                    return null;
                }

                byteArray[i] = (byte)temp;

            }
            return byteArray;
        }

        /// <summary>
        /// Log Export有効時の書き込み関数
        /// </summary>
        /// <param name="text"></param>
        private void ExportLog(string text, bool timeStamp = true, bool newLine = true)
        {
            if (this.checkBoxLogExport.Checked)
            {
                if (timeStamp)
                {
                    if (newLine)
                    {
                        File.AppendAllText(logFilePath, DateTime.Now.ToString().Replace("/", "-").Replace(" ", "_") + "," + text + Environment.NewLine);
                    }
                    else
                    {
                        File.AppendAllText(logFilePath, DateTime.Now.ToString().Replace("/", "-").Replace(" ", "_") + "," + text);
                    }

                }
                else
                {
                    if (newLine)
                    {
                        File.AppendAllText(logFilePath, text + Environment.NewLine);
                    }
                    else
                    {
                        File.AppendAllText(logFilePath, text);
                    }
                }
            }

        }

        /// <summary>
        /// 書き込みモードでファイルを開けるか確認
        /// </summary>
        /// <param name="filePath"></param>
        /// <returns></returns>
        private bool LogFileAccesability(string filePath)
        {
            try
            {
                // 書き込みモードでファイルを開けるか確認
                using (FileStream fp = File.Open(filePath, FileMode.Open, FileAccess.Write))
                {
                    // 開ける
                    return true;
                }
            }
            catch
            {
                // 開けない
                return false;
            }
        }
        private void RestartButton_Click(object sender, EventArgs e)
        {
            // デバイス初期化
            SVIControl.DeviceOpen(boardInformation.DeviceId, boardInformation.BoardType);

            SVIControl.RestartBoard();
            Close();
        }

        #endregion

        #region ▼ Public Method
        #endregion

        #region ▼ Override Method
        #endregion



    }
}
